@@ -1,6 +1,6 @@
-// $Id: blacklist.c,v 1.7 2002/11/27 21:16:05 ensc Exp $    --*- c++ -*--
+// $Id: blacklist.c,v 1.10 2003/05/27 11:31:07 ensc Exp $    --*- c++ -*--
 
-// Copyright (C) 2002 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
+// Copyright (C) 2002,2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
 //  
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -54,6 +54,7 @@ struct NetData
     struct in_addr	ip;
     struct in_addr	mask;
     BlackListStatus	status;
+    struct ether_addr	mac;
 };
 
 static unsigned int
@@ -108,8 +109,8 @@ NetData_sortCompare(void const *lhs_v, void const *rhs_v)
   if      (result!=0) {}
   else if (ntohl(lhs->mask.s_addr) < ntohl(rhs->mask.s_addr)) result = -1;
   else if (ntohl(lhs->mask.s_addr) > ntohl(rhs->mask.s_addr)) result = +1;
-  else if (ntohl(lhs->ip.s_addr) < ntohl(rhs->ip.s_addr))     result = -1;
-  else if (ntohl(lhs->ip.s_addr) > ntohl(rhs->ip.s_addr))     result = +1;
+  else if (ntohl(lhs->ip.s_addr)   < ntohl(rhs->ip.s_addr))   result = -1;
+  else if (ntohl(lhs->ip.s_addr)   > ntohl(rhs->ip.s_addr))   result = +1;
   else result = lhs->status - rhs->status;
 #endif  
 
@@ -124,12 +125,28 @@ BlackList_init(BlackList *lst, char const *filename)
   Vector_init(&lst->ip_list,  sizeof(struct IPData));
   Vector_init(&lst->net_list, sizeof(struct NetData));
 
-  lst->filename   = filename;
+  lst->filename   = strdup(filename);
   lst->last_mtime = 0;
 
   BlackList_update(lst);
 }
 
+#if ENSC_TESTSUITE
+void
+BlackList_free(BlackList *lst)
+{
+  assert(lst!=0);
+
+  free(const_cast(char *)(lst->filename));
+  Vector_free(&lst->ip_list);
+  Vector_free(&lst->net_list);
+
+#ifndef NDEBUG
+  lst->filename = (void *)(0xdeadbeef);
+#endif
+}
+#endif
+
 static bool
 BlackList_parseLine(BlackList *lst, char *start, char const *end, size_t line_nr)
 {
@@ -229,11 +246,6 @@ BlackList_parseLine(BlackList *lst, char *start, char const *end, size_t line_nr
     if (parse_status!=blIGNORE) parse_status = blRAND;
   }
   else {
-    if (has_mask) {
-      writeUInt(2, line_nr);
-      WRITE_MSGSTR(2, ": MAC address given for a net-address\n");
-      return false;
-    }
     for (pos=start; pos<end; ++pos) {
       switch (*pos) {
 	case '#'	:
@@ -267,6 +279,7 @@ BlackList_parseLine(BlackList *lst, char *start, char const *end, size_t line_nr
     data->ip     = parse_ip;
     data->mask   = parse_mask;
     data->status = parse_status;
+    data->mac	 = parse_mac;
 
     data->ip.s_addr &= data->mask.s_addr;
   }
@@ -393,6 +406,8 @@ struct ether_addr const *
 BlackList_getMac(BlackList const *lst_const, struct in_addr const ip, struct ether_addr *res)
 {
   struct ether_addr const *	result = 0;
+    // 'status' is tied to 'result'; therefore compiler warnings about possible uninitialized usage
+    // can be ignored.
   BlackListStatus		status;
     // C does not allow Vector_begin()/end() functions accepting both const and non-const
     // parameters. To prevent const_cast'ing in every call to these functions, const_cast the
@@ -416,7 +431,7 @@ BlackList_getMac(BlackList const *lst_const, struct in_addr const ip, struct eth
     for (; data!=end_ptr; ++data) {
       if ((ip.s_addr & data->mask.s_addr) == data->ip.s_addr) {
 	status = data->status;
-	*res   = DEFAULT_MAC;
+	*res   = data->mac;
 	result = res;
 	break;
       }
@@ -475,6 +490,8 @@ BlackList_print(BlackList *lst, int fd)
 
     for (i =Vector_begin(&lst->net_list);
 	 i!=Vector_end(&lst->net_list); ++i) {
+      char *		aux = ether_ntoa(&i->mac);
+
       switch (i->status) {
 	case blUNDECIDED	:  write(fd, "?", 1); break;
 	case blIGNORE		:  write(fd, "!", 1); break;
@@ -486,6 +503,8 @@ BlackList_print(BlackList *lst, int fd)
       writeIP(fd, i->ip);
       WRITE_MSGSTR(fd, "/");
       writeIP(fd, i->mask);
+      WRITE_MSGSTR(fd, "\t\t");
+      WRITE_MSG(fd, aux);
       WRITE_MSGSTR(fd, "\n");
     }
   }
