@@ -1,6 +1,6 @@
-// $Id: vector.c,v 1.4 2002/11/27 20:48:22 ensc Exp $    --*- c++ -*--
+// $Id: vector.c,v 1.6 2003/05/27 11:31:07 ensc Exp $    --*- c++ -*--
 
-// Copyright (C) 2002 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
+// Copyright (C) 2002,2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
 //  
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -40,6 +40,22 @@ Vector_init(struct Vector *vec, size_t elem_size)
   vec->allocated = 0;
 }
 
+#if ENSC_TESTSUITE
+void
+Vector_free(struct Vector *vec)
+{
+  assert(vec!=0);
+  free(vec->data);
+
+#ifndef NDEBUG  
+  vec->count     = 0xdeadbeef;
+  vec->allocated = 0xdeadbeef;
+  vec->elem_size = 0xdeadbeef;
+  vec->data      = (void *)(0xdeadbeef);
+#endif
+}
+#endif
+
 void *
 Vector_search(struct Vector *vec, void const *key,
 	      int (*compare)(const void *, const void *))
